/*
 * Po drate 4 Lemmings clone
 * Copyright (C) 2009 Jiri Benc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _LEMMINGS_H
#define _LEMMINGS_H

#define XSIZE	80
#define YSIZE	24

extern char level[YSIZE][XSIZE + 1];

struct lemming;

struct skill {
	int (*activate)(struct lemming *lem);
	int (*move)(struct lemming *lem);
	void (*wall)(struct lemming *lem);
	int (*pre_fall)(struct lemming *lem);
	void (*post_fall)(struct lemming *lem);
};

struct lemming {
	int x, y, dir;
	int fall;
	int skill_data;
	char *draw;
	struct skill *skill;
	struct lemming *prev, *next;
};

#define SKILLS	8
extern struct skill skills[];
// skill (SKILLS + 1) is a dying lemming

#define EMPTY(x, y)	({ char c = level[y][x]; c == ' ' || c == 0 || c == '(' || c == ')' || c == 'T'; })
#define BLOCKER(x, y)	({ char c = level[y][x]; c == 'T'; })
#define FINISH(x, y)	({ char c = level[y][x]; c == '(' || c == ')'; })

#define CHECK_BORDERS(lem, xrel, yrel)	\
				(((lem)->x + (xrel)) > 0 && ((lem)->x + (xrel)) < XSIZE - 1 && \
				 ((lem)->y + (yrel)) > 0 && ((lem)->y + (yrel)) < YSIZE - 1)
#define DIR_TO_XREL(lem)	(1 - (lem)->dir * 2)

// lemmings.c
void draw_lemming(struct lemming *lem);
void erase_lemming(struct lemming *lem);
void move_lemming(struct lemming *lem, int xrel, int yrel);
void highlight_lemming(struct lemming *lem, char c);
void kill_lemming(struct lemming *lem);
void delete_lemming(struct lemming *lem);
void modify_level(int x, int y, char c);

void adjust_speed(int rel);
int odd_step(void);

void fatal(char *msg);

// program.c
void get_program(void);
void advance_program(void);
int program_get_skill(void);

#endif
